@@ -1,9 +1,9 @@
-import pkg_resources
+from importlib.metadata import version as pkg_ver
 
 name = "nb2workflow"
 
 def version(print_it=True):
-    v = pkg_resources.get_distribution("nb2workflow").version
+    v = pkg_ver("nb2workflow")
     if print_it:
         print(v)
 
