@@ -133,15 +133,8 @@ class NBRepo:
                 "last_change_time": meta['last_change_time'],
                 "resources": meta["resources"]}
     
-    @lru_cache
-    def generate_dockerfile(self,
-                            source_from: str, 
-                            nb2wversion: str) -> str:
-        if source_from not in ['localdir', 'git']:
-            raise NotImplementedError('Unsupported source code location %s', source_from)
-        
-        tmpl = jenv.get_template('Dockerfile.jinja')
-            
+    @cached_property
+    def mmoda_config(self) -> dict:
         config_fn = self.local_repo_path / "mmoda.yaml"
 
         config = default_config.copy()
@@ -155,29 +148,41 @@ class NBRepo:
             logger.info("no extra config in %s", config_fn)   
         logger.info("complete config: %s", config)
 
+        return config
 
-        if config['use_repo_base_image']: 
+
+    @lru_cache
+    def generate_dockerfile(self,
+                            source_from: str, 
+                            nb2wversion: str) -> str:
+        if source_from not in ['localdir', 'git']:
+            raise NotImplementedError('Unsupported source code location %s', source_from)
+        
+        tmpl = jenv.get_template('Dockerfile.jinja')
+            
+        if self.mmoda_config['use_repo_base_image']: 
             with open(self.context_dir / "Dockerfile", "r") as fd:
                 dockerfile_base = fd.read()
         else:
             dockerfile_base = None
         
-        notebook_fullpath_in_container = pathlib.Path('/repo') / (config['notebook_path'].strip("/"))
+        notebook_fullpath_in_container = pathlib.Path('/repo') / (self.mmoda_config['notebook_path'].strip("/"))
         logger.info("using notebook_fullpath_in_container: %s", notebook_fullpath_in_container)
 
         # will be used only if not use_repo_base_image but need to define to pass into template
         has_conda_env = False
         inject_python_version_str = ''
       
-        if not config['use_repo_base_image']:
-            inject_python_version_str = f"sed -i '/dependencies/a \ \ - python={default_python_version}' /repo/environment.yml"
+        if not self.mmoda_config['use_repo_base_image']:
+            inject_python_version_str = f"/tmp/yq -i '.dependencies += \"python={default_python_version}\"' /repo/environment.yml"
             if os.path.exists( self.local_repo_path / 'environment.yml' ):
                 with open(self.local_repo_path / 'environment.yml') as fd:
                     parsed_env = yaml.safe_load(fd)
-                    if 'dependencies' in parsed_env:
+                    dependencies_parsed_env = parsed_env.get('dependencies', None)
+                    if dependencies_parsed_env is not None:
                         has_conda_env = True
                         match_spec = re.compile(r'^python[~=<> ]')
-                        for dep in parsed_env['dependencies']:
+                        for dep in dependencies_parsed_env:
                             if isinstance(dep, str) and match_spec.match(dep):
                                 inject_python_version_str = f'echo "Using {dep}"'
                                 break
@@ -197,7 +202,7 @@ class NBRepo:
             nb2w_version_spec = nb2w_version_spec,
             metadata = self.pre_build_metadata,
             nbpath = notebook_fullpath_in_container,
-            filename_pattern = config['filename_pattern']
+            filename_pattern = self.mmoda_config['filename_pattern']
         )
         
         with open(self.context_dir / "Dockerfile", "w") as fd:
@@ -209,8 +214,15 @@ class NBRepo:
     def resource_requirements(self) -> dict[str, dict]:
         resources = {}
 
-        search_pattern = os.path.join(self.context_dir,'**/*.ipynb')
-        for nb_file in glob.glob(search_pattern, recursive=True):
+        search_pattern = os.path.join(self.context_dir, 'nb-repo', self.mmoda_config['notebook_path'].strip(os.sep), '*.ipynb')
+        notebooks = glob.glob(search_pattern)
+
+        for nb_file in notebooks:
+            if not re.match(self.mmoda_config['filename_pattern'], os.path.basename(nb_file)):
+                logger.info(f"notebook {nb_file} doesn't match filename pattern: skipping")
+                continue
+            logger.info(f'Analysing notebook for requirements: {nb_file}')
+
             nba = NotebookAdapter(nb_file)
             g = nba._graph
             for r in self.ontology.get_requested_resources(g):
@@ -416,9 +428,10 @@ class NBRepo:
 
         secretenv = []
         for name, resource in use_container_meta['resources'].items():
-            verify_resource_secret(name, resource['required'], namespace=namespace)
-            for env in resource['env_vars']:
-                secretenv.append((env, name))
+            secret_is_defined = verify_resource_secret(name, resource['required'], namespace=namespace)
+            if secret_is_defined:
+                for env in resource['env_vars']:
+                    secretenv.append((env, name))
 
         os.makedirs(self.context_dir/'deploy', exist_ok=True)
 
@@ -524,6 +537,8 @@ def verify_resource_secret(name, required, namespace="oda-staging"):
         raise RuntimeError(message)
     else:
         logger.warning(message)
+    
+    return False
 
 def deploy_k8s(container_info, 
            deployment_base_name, 
