@@ -53,6 +53,9 @@ from git import Repo, InvalidGitRepositoryError, GitCommandError
 import logging
 from threading import Lock
 
+from nbclient.exceptions import DeadKernelError
+
+
 logger=logging.getLogger(__name__)
 
 logstasher = logstash.LogStasher()
@@ -142,7 +145,16 @@ def cast_parameter(x,par):
         if par.get('is_optional', False):
             return None
         else:
-            raise ValueError(f'Non-optional parameter is set to None')
+            raise ValueError(f'Non-optional parameter {par["name"]} is set to None')
+    # For all parameters, empty value is represented with None value, as the empty string is distinct from None, and this approach doesn't directly cause any problems.
+    # With POSIXPath parameters, we had a problem / unintuitive behaviour due to how file parameters are passed from frontend.
+    # Both empty string and None intuitively have clear meaning of "no file" in this case.
+    # To align with the same behaviour of all the other parameters,
+    # we only support explicit None and don't allow empty string for POSIXPath at all.
+    # This will also simplify the needed adaptations in the dispatcher, otherwise it's hard to guess there what is expected to represent emptiness.
+    # It is also extended to the whole FileReference class, not only POSIXPath just for uniformness.
+    if x == '' and par.get('is_file_reference', False):
+        raise ValueError(f'Empty string is not a valid value of FileReference parameter {par["name"]}')
     return par['python_type'](x)
 
 
@@ -200,21 +212,22 @@ def reconcile_python_type(value: Any,
                           type_annotation: str | type[T] | None = None, 
                           owl_type: str | None = None, 
                           extra_ttl: str | None = None, 
-                          name: str = '') -> tuple[type, bool]:
+                          name: str = '') -> tuple[type, bool, bool]:
     '''
     Reconcile python type of the default value with type and owl annotations
     We expect ~json here, so basically int, float, str, list, dict or None
     Respects duck typing: if default is int and float is allowed, returns float
     '''
-
+    
     if type_annotation is None and owl_type is None:
         if value is not None:
-            return type(value), False
+            return type(value), False, False
         else:
             raise TypeCheckError(f"Default value of the required parameter {name} isn't defined.")
 
     owl_dt = None
     is_optional_owl = False
+    is_file_reference = False
     if owl_type is not None:
         if extra_ttl is None: 
             extra_ttl = ''
@@ -223,6 +236,12 @@ def reconcile_python_type(value: Any,
         if xsd_dt:
             owl_dt = xsd_type_to_python_type(xsd_dt)
         is_optional_owl = ontology.is_optional(owl_type)
+        # refer tpo the comment in the cast_parameter function
+        is_file_reference = 'http://odahub.io/ontology#FileReference' in ontology.get_parameter_hierarchy(owl_type)
+        if is_file_reference and value == '':
+            raise TypeCheckError(f"Empty string value is not permitted for parameter {name} of type FileReference. "
+                                  "Please use None and annotate parameter as optional instead.")
+
 
     is_optional_hint = False
     hint_fref = None
@@ -250,7 +269,7 @@ def reconcile_python_type(value: Any,
     # So other will fail checking value, but it's OK.
     if value is None:
         if not is_optional_owl and not is_optional_hint:
-            raise TypeCheckError(f"Required parameter {name} shouldn't be None.")
+            raise TypeCheckError(f"Non-optional parameter {name} can't be None.")
         elif owl_dt is None and hint_fref is None:
             raise TypeCheckError(f"Default value of the parameter {name} can't be defined.")
         else:
@@ -261,7 +280,7 @@ def reconcile_python_type(value: Any,
                 except TypeCheckError:
                     pass
                 else:
-                    return type(ex), True
+                    return type(ex), True, is_file_reference
             raise TypeCheckError(f"No possible type is found for the parameter {name}.")
     elif isinstance(value, int) and not isinstance(value, bool):
         # be permissive if float is possible
@@ -270,10 +289,10 @@ def reconcile_python_type(value: Any,
         except TypeCheckError:
             pass
         else:
-            return float, is_optional_owl or is_optional_hint
+            return float, is_optional_owl or is_optional_hint, False
         
         check_type_both(value)
-        return int, is_optional_owl or is_optional_hint
+        return int, is_optional_owl or is_optional_hint, False
     elif isinstance(value, bool):
         check_type_both(value)
         try:
@@ -282,10 +301,10 @@ def reconcile_python_type(value: Any,
             pass
         else:
             raise TypeCheckError(f"Boolean parameter {name} is annotated as integer.")
-        return type(value), is_optional_owl or is_optional_hint
+        return type(value), is_optional_owl or is_optional_hint, False
     else:
         check_type_both(value)
-        return type(value), is_optional_owl or is_optional_hint
+        return type(value), is_optional_owl or is_optional_hint, is_file_reference
 
 
 
@@ -299,6 +318,7 @@ class InputParameter:
     owl_type: Optional[str] = None
     extra_ttl: Optional[str] = None
     is_optional: bool = False
+    is_file_reference: bool = False
 
     def as_dict(self):
         return asdict(self)
@@ -307,7 +327,6 @@ class InputParameter:
 class NotebookAdapter:
     limit_output_attachment_file = None
 
-
     def __init__(self, notebook_fn, tempdir_cache=None, n_download_max_tries=10, download_retry_sleep_s=.5, max_download_size=500e6):
         self.notebook_fn = os.path.abspath(notebook_fn)
         self.name = notebook_short_name(notebook_fn)
@@ -315,6 +334,7 @@ class NotebookAdapter:
         self._graph = rdflib.Graph()
         logger.debug("notebook adapter for %s", self.notebook_fn)
         logger.debug(self.extract_parameters())
+        logger.debug(self.extract_output_declarations())
         self.n_download_max_tries = n_download_max_tries
         self.download_retry_sleep_s = download_retry_sleep_s
         self.max_download_size = max_download_size
@@ -496,11 +516,12 @@ class NotebookAdapter:
             parsed_comment = understand_comment_references(par_detail['comment'],
                                                            fallback_type=fallback_type)
             
-            python_type, is_optional = reconcile_python_type(par_detail['value'],
-                                            type_annotation=par_detail['type_annotation'],
-                                            owl_type=parsed_comment.get('owl_type', None),
-                                            extra_ttl=parsed_comment.get('extra_ttl', None),
-                                            name = par_detail['varname'])
+            python_type, is_optional, is_file_reference = reconcile_python_type(
+                par_detail['value'],
+                type_annotation=par_detail['type_annotation'],
+                owl_type=parsed_comment.get('owl_type', None),
+                extra_ttl=parsed_comment.get('extra_ttl', None),
+                name = par_detail['varname'])
             
             par = InputParameter(raw_line = par_detail['raw_line'],
                                  name = par_detail['varname'],
@@ -509,7 +530,8 @@ class NotebookAdapter:
                                  comment = par_detail['comment'],
                                  owl_type = parsed_comment.get('owl_type', None),
                                  extra_ttl = parsed_comment.get('extra_ttl', None),
-                                 is_optional=is_optional)
+                                 is_optional=is_optional,
+                                 is_file_reference=is_file_reference)
             
             # This leads to some recursion, but it's not really used anywhere. 
             # TODO: integrate with ontology.function_semantic_signature
@@ -533,7 +555,7 @@ class NotebookAdapter:
     def extract_parameters(self):
         nb = self.read()
 
-        self.input_parameters = {}
+        self._input_parameters = {}
         self.system_parameters = {}
                
         for cell in nb.cells:
@@ -547,7 +569,18 @@ class NotebookAdapter:
                     pars = {**getattr(self, attr), **pars}
                     setattr(self, attr, pars)
 
-        return self.input_parameters
+        return self._input_parameters
+
+    @property
+    def input_parameters(self):
+        try:
+            return self._input_parameters
+        except AttributeError:
+            return self.extract_parameters()
+
+    @input_parameters.setter
+    def input_parameters(self, value):
+        self._input_parameters = value
 
     @cached_property
     def token_access(self):
@@ -632,6 +665,7 @@ class NotebookAdapter:
         return exceptions
 
     def _execute(self, parameters, progress_bar=True, log_output=True, inplace=False, context={}, tmpdir_key=None):
+        exceptions = []
 
         if not inplace :
             tmpdir = self.new_tmpdir(tmpdir_key)
@@ -649,26 +683,26 @@ class NotebookAdapter:
                 if 'git-lfs' in str(e):
                     # this error may occur if the repo was originally cloned by the different version of git utility
                     # e.g. when repo is mounted with docker run -v
-                    raise Exception("We got some problem cloning the repository, the problem seems to be related to git-lfs. You might want to try reinitializing git-lfs with 'git-lfs install; git-lfs pull'")
+                    exceptions.append(RuntimeError("We got some problem cloning the repository, the problem seems to be related to git-lfs. You might want to try reinitializing git-lfs with 'git-lfs install; git-lfs pull'"))
                 else:
-                    raise e
+                    exceptions.append(e)
         else:
             tmpdir =os.path.dirname(os.path.realpath(self.notebook_fn))
             logger.info("executing inplace, no tmpdir is input dir: %s", tmpdir)
 
         r = self.handle_url_params(parameters, tmpdir, context=context)
 
-        if len(context) > 0:
-            self._pass_context(tmpdir, context)
-
-        self.update_summary(state="started", parameters=parameters)
-
-        self.inject_output_gathering()
-        exceptions = []
-
-        if len(r['exceptions']) > 0:
+        if len(r['exceptions'])>0:
             exceptions.extend(r['exceptions'])
-        else:
+        
+        if len(exceptions) == 0:
+            if len(context) > 0:
+                self._pass_context(tmpdir, context)
+
+            self.update_summary(state="started", parameters=parameters)
+
+            self.inject_output_gathering()
+
             ntries = 10
             while ntries > 0:
                 try:
@@ -684,22 +718,29 @@ class NotebookAdapter:
                        log_output = True,
                        cwd = tmpdir,
                     )
-                except pm.PapermillExecutionError as e:
+                except (pm.PapermillExecutionError, DeadKernelError) as e:
                     exceptions.append([e,e.args])
                     logger.info(e)
                     logger.info(e.args)
 
-                    if e.ename == "WorkflowIncomplete":
+                    if isinstance(e, DeadKernelError):
+                        sentry.capture_exception(e)
+                        
+                    elif e.ename == "WorkflowIncomplete":
                         logger.info("detected incomplete workflow")
                         self.update_summary(state="incomplete dependency", dependency=repr(e))
                         raise  PapermillWorkflowIncomplete()
-
+                
                 except nbformat.reader.NotJSONError:
                     ntries -= 1
                     logger.info("retrying... %s", ntries)
                     time.sleep(2)
                     continue
-
+                
+                except Exception as e:
+                    logger.error('Unexpected exception %s', e)
+                    sentry.capture_exception(e)
+                    
                 break
 
         if len(exceptions) == 0:
@@ -837,27 +878,39 @@ class NotebookAdapter:
 
         return file_name
 
-    def handle_url_params(self, parameters, tmpdir, context={}):
-        adapted_parameters = copy.deepcopy(parameters)
-        exceptions = []
+    @staticmethod
+    def check_is_file_input(owl_type: str) -> dict[str, bool]:
         posix_path_with_annotations_pattern = re.compile(rf"^{re.escape(oda_prefix)}.*_POSIXPath_")
         file_url_with_annotations_pattern = re.compile(rf"^{re.escape(oda_prefix)}.*_FileURL_")
         file_reference_with_annotations_pattern = re.compile(rf"^{re.escape(oda_prefix)}.*_FileReference_")
+
+        if ontology.is_ontology_available:
+            parameter_hierarchy = ontology.get_parameter_hierarchy(owl_type)
+            is_posix_path = f"{oda_prefix}POSIXPath" in parameter_hierarchy
+            is_file_url = f"{oda_prefix}FileURL" in parameter_hierarchy
+            is_file_reference = f"{oda_prefix}FileReference" in parameter_hierarchy
+        else:
+            is_posix_path = f"{oda_prefix}POSIXPath" == owl_type or \
+                            posix_path_with_annotations_pattern.match(owl_type) is not None
+            is_file_url = f"{oda_prefix}FileURL" == owl_type or \
+                            file_url_with_annotations_pattern.match(owl_type) is not None
+            is_file_reference = f"{oda_prefix}FileReference" == owl_type or \
+                                file_reference_with_annotations_pattern.match(owl_type) is not None
+        return {
+            'is_posix_path': is_posix_path,
+            'is_file_url': is_file_url,
+            'is_file_reference': is_file_reference
+        }
+
+    def handle_url_params(self, parameters, tmpdir, context={}):
+        adapted_parameters = copy.deepcopy(parameters)
+        exceptions = []
         for input_par_name, input_par_obj in self.input_parameters.items():
-            if ontology.is_ontology_available:
-                parameter_hierarchy = ontology.get_parameter_hierarchy(input_par_obj['owl_type'])
-                is_posix_path = f"{oda_prefix}POSIXPath" in parameter_hierarchy
-                is_file_url = f"{oda_prefix}FileURL" in parameter_hierarchy
-                is_file_reference = f"{oda_prefix}FileReference" in parameter_hierarchy
-            else:
-                is_posix_path = f"{oda_prefix}POSIXPath" == input_par_obj['owl_type'] or \
-                                posix_path_with_annotations_pattern.match(input_par_obj['owl_type']) is not None
-                is_file_url = f"{oda_prefix}FileURL" == input_par_obj['owl_type'] or \
-                                file_url_with_annotations_pattern.match(input_par_obj['owl_type']) is not None
-                is_file_reference = f"{oda_prefix}FileReference" == input_par_obj['owl_type'] or \
-                                    file_reference_with_annotations_pattern.match(input_par_obj['owl_type']) is not None
+            is_posix_path, is_file_url, is_file_reference = self.check_is_file_input(input_par_obj['owl_type']).values()
+
             if is_posix_path or is_file_url or is_file_reference:
                 arg_par_value = parameters.get(input_par_name, None)
+
                 if arg_par_value is None:
                     arg_par_value = input_par_obj['default_value']
                 if validators.url(arg_par_value, simple_host=True):
